@@ -410,7 +410,7 @@ lookup_cd (SjMetadata *metadata)
           album->artist = g_strdup (_("Unknown Artist"));
         }
       }
-      if (data && mb_GetResultData(priv->mb, MBE_AlbumGetAlbumArtistSortName, data, sizeof (data))) {
+      if (mb_GetResultData(priv->mb, MBE_AlbumGetAlbumArtistSortName, data, sizeof (data))) {
         album->artist_sortname = g_strdup (data);
       }
     }
@@ -505,6 +505,12 @@ lookup_cd (SjMetadata *metadata)
       convert_encoding(&album->artist_sortname);
     }
 
+    if (from_freedb) {
+      album->metadata_source = SOURCE_FREEDB;
+    } else {
+      album->metadata_source = SOURCE_MUSICBRAINZ;
+    }
+
     albums = g_list_append (albums, album);
 
     mb_Select (priv->mb, MBS_Rewind);
