@@ -52,16 +52,20 @@
 #include "sj-main.h"
 #include "sj-prefs.h"
 #include "sj-play.h"
+#include "sj-genres.h"
+#include "gedit-message-area.h"
 #include "bacon-volume.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 
 static void reread_cd (gboolean ignore_no_media);
 static void update_ui_for_album (AlbumDetails *album);
+static void set_duplication (gboolean enable);
 
 /* Prototypes for the signal blocking/unblocking in update_ui_for_album */
 void on_title_edit_changed(GtkEditable *widget, gpointer user_data);
 void on_artist_edit_changed(GtkEditable *widget, gpointer user_data);
+void on_year_edit_changed(GtkEditable *widget, gpointer user_data);
 
 GladeXML *glade;
 
@@ -71,21 +75,27 @@ SjExtractor *extractor;
 GConfClient *gconf_client;
 
 GtkWidget *main_window;
-static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
+static GtkWidget *message_area_vbox;
+static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry, *year_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
 static GtkWidget *extract_menuitem, *play_menuitem, *next_menuitem, *prev_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 static GtkWidget *submit_menuitem;
+static GtkWidget *duplicate;
 GtkListStore *track_store;
 static BaconMessageConnection *connection;
 GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
 
+GtkWidget *current_message_area;
+
 const char *path_pattern, *file_pattern;
 char *base_uri;
 NautilusBurnDrive *drive = NULL;
 gboolean strip_chars;
 gboolean eject_finished;
+gboolean open_finished;
 gboolean extracting = FALSE;
+static gboolean duplication_enabled;
 
 static gint total_no_of_tracks;
 static gint no_of_tracks_selected;
@@ -244,7 +254,6 @@ void on_eject_activate (GtkMenuItem *item, gpointer user_data)
   /* first make sure we're not playing */
   stop_playback ();
 
-  nautilus_burn_drive_unlock (drive);
   nautilus_burn_drive_eject (drive);
 }
 
@@ -346,6 +355,126 @@ static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
   }
 }
 
+/* Taken from gedit */
+static void
+set_message_area_text_and_icon (GeditMessageArea *message_area,
+				const gchar   *icon_stock_id,
+				const gchar   *primary_text,
+				const gchar   *secondary_text)
+{
+  GtkWidget *hbox_content;
+  GtkWidget *image;
+  GtkWidget *vbox;
+  gchar *primary_markup;
+  gchar *secondary_markup;
+  GtkWidget *primary_label;
+  GtkWidget *secondary_label;
+
+  hbox_content = gtk_hbox_new (FALSE, 8);
+  gtk_widget_show (hbox_content);
+
+  image = gtk_image_new_from_stock (icon_stock_id, GTK_ICON_SIZE_DIALOG);
+  gtk_widget_show (image);
+  gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
+  gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
+
+  vbox = gtk_vbox_new (FALSE, 6);
+  gtk_widget_show (vbox);
+  gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
+
+  primary_markup = g_markup_printf_escaped ("<b>%s</b>", primary_text);
+  primary_label = gtk_label_new (primary_markup);
+  g_free (primary_markup);
+  gtk_widget_show (primary_label);
+  gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
+  gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
+  gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
+
+  if (secondary_text != NULL) {
+    secondary_markup = g_markup_printf_escaped ("<small>%s</small>",
+						secondary_text);
+    secondary_label = gtk_label_new (secondary_markup);
+    g_free (secondary_markup);
+    gtk_widget_show (secondary_label);
+    gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
+    gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
+    gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
+    gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
+  }
+
+  gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
+				   hbox_content);
+}
+
+/* Taken from gedit */
+static void
+set_message_area (GtkWidget *container,
+		  GtkWidget *message_area)
+{
+  if (current_message_area == message_area)
+      return;
+
+  if (current_message_area != NULL)
+    gtk_widget_destroy (current_message_area);
+
+  current_message_area = message_area;
+
+  if (message_area == NULL)
+    return;
+
+  gtk_box_pack_start (GTK_BOX (container),
+		      current_message_area,
+		      FALSE,
+		      FALSE,
+		      0);
+
+  g_object_add_weak_pointer (G_OBJECT (current_message_area),
+			     (gpointer)&current_message_area);
+}
+
+static GtkWidget* musicbrainz_submit_message_area_new (char *title,
+						       char *artist)
+{
+  GtkWidget *message_area;
+  char *primary_text;
+
+  g_return_val_if_fail (title != NULL, NULL);
+  g_return_val_if_fail (artist != NULL, NULL);
+
+  message_area = gedit_message_area_new ();
+
+  gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+			      _("_Submit Album"),
+			      GTK_RESPONSE_OK);
+  gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+			      GTK_STOCK_CANCEL,
+			      GTK_RESPONSE_CANCEL);
+
+  /* Translators: title, artist */
+  primary_text = g_strdup_printf (_("Could not find %s by %s on MusicBrainz."), title, artist);
+
+  set_message_area_text_and_icon (GEDIT_MESSAGE_AREA (message_area),
+				  "gtk-dialog-info",
+				  primary_text,
+				  _("You can improve the MusicBrainz database by adding this album."));
+
+  g_free (primary_text);
+
+  return message_area;
+}
+
+static void musicbrainz_submit_message_area_response (GeditMessageArea *message_area,
+						      gint           response_id,
+						      gpointer       user_data)
+{
+  if (response_id == GTK_RESPONSE_OK) {
+    on_submit_activate (NULL, NULL);
+  }
+
+  set_message_area (message_area_vbox, NULL);
+}
+
 /**
  * Utility function to update the UI for a given Album
  */
@@ -364,10 +493,12 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_entry_set_text (GTK_ENTRY (title_entry), "");
     gtk_entry_set_text (GTK_ENTRY (artist_entry), "");
     gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
+    gtk_entry_set_text (GTK_ENTRY (year_entry), "");
     gtk_label_set_text (GTK_LABEL (duration_label), "");
     gtk_widget_set_sensitive (title_entry, FALSE);
     gtk_widget_set_sensitive (artist_entry, FALSE);
     gtk_widget_set_sensitive (genre_entry, FALSE);
+    gtk_widget_set_sensitive (year_entry, FALSE);
     gtk_widget_set_sensitive (play_button, FALSE);
     gtk_widget_set_sensitive (play_menuitem, FALSE);
     gtk_widget_set_sensitive (extract_button, FALSE);
@@ -376,21 +507,30 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
     gtk_widget_set_sensitive (prev_menuitem, FALSE);
     gtk_widget_set_sensitive (next_menuitem, FALSE);
+    set_duplication (FALSE);
+
+    set_message_area (message_area_vbox, NULL);
   } else {
     gtk_list_store_clear (track_store);
 
     g_signal_handlers_block_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_block_by_func (artist_entry, on_artist_edit_changed, NULL);
+    g_signal_handlers_block_by_func (year_entry, on_year_edit_changed, NULL);
     gtk_entry_set_text (GTK_ENTRY (title_entry), album->title);
     gtk_entry_set_text (GTK_ENTRY (artist_entry), album->artist);
+    if (g_date_valid (album->release_date)) {
+      gtk_entry_set_text (GTK_ENTRY (year_entry), g_strdup_printf ("%d", g_date_get_year (album->release_date)));
+    }
     g_signal_handlers_unblock_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_unblock_by_func (artist_entry, on_artist_edit_changed, NULL);
+    g_signal_handlers_unblock_by_func (year_entry, on_year_edit_changed, NULL);
     /* Clear the genre field, it's from the user */
     gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
 
     gtk_widget_set_sensitive (title_entry, TRUE);
     gtk_widget_set_sensitive (artist_entry, TRUE);
     gtk_widget_set_sensitive (genre_entry, TRUE);
+    gtk_widget_set_sensitive (year_entry, TRUE);
     gtk_widget_set_sensitive (play_button, TRUE);
     gtk_widget_set_sensitive (play_menuitem, TRUE);
     gtk_widget_set_sensitive (extract_button, TRUE);
@@ -399,6 +539,7 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
     gtk_widget_set_sensitive (prev_menuitem, FALSE);
     gtk_widget_set_sensitive (next_menuitem, FALSE);
+    set_duplication (TRUE);
     
     for (l = album->tracks; l; l=g_list_next (l)) {
       GtkTreeIter iter;
@@ -426,12 +567,31 @@ static void update_ui_for_album (AlbumDetails *album)
     } else {
       gtk_label_set_text (GTK_LABEL (duration_label), _("(unknown)"));
     }
+
+    /* If album details don't come from MusicBrainz ask user to add them */
+    if (album->metadata_source != SOURCE_MUSICBRAINZ) {
+      GtkWidget *message_area;
+
+      message_area = musicbrainz_submit_message_area_new (album->title,
+							  album->artist);
+
+      set_message_area (message_area_vbox, message_area);
+
+      g_signal_connect (message_area,
+			"response",
+			G_CALLBACK (musicbrainz_submit_message_area_response),
+			NULL);
+
+      gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (message_area),
+					    GTK_RESPONSE_CANCEL);
+
+      gtk_widget_show (message_area);
+    }
   }
 }
 
 /**
- * Called by the Multiple Album dialog when the user hits return in
- * the list view
+ * Callback that gets fired when a user double clicks on a row
  */
 static void album_row_activated (GtkTreeView *treeview,
                                  GtkTreePath *arg1,
@@ -442,6 +602,22 @@ static void album_row_activated (GtkTreeView *treeview,
   gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
+/**
+ * Callback that gets fired when an the selection changes. We use this to
+ * change the sensitivity of the continue button
+ */
+static void selected_album_changed (GtkTreeSelection *selection, 
+                                    gpointer *user_data)
+{
+  GtkWidget *ok_button = GTK_WIDGET (user_data);
+
+  if (gtk_tree_selection_get_selected (selection, NULL, NULL)) {
+    gtk_widget_set_sensitive (ok_button, TRUE);
+  } else {
+    gtk_widget_set_sensitive (ok_button, FALSE);
+  }
+}
+
 /**
  * Utility function for when there are more than one albums available
  */
@@ -453,6 +629,7 @@ AlbumDetails* multiple_album_dialog(GList *albums)
   AlbumDetails *album;
   GtkTreeIter iter;
   int response;
+  GtkWidget *ok_button = NULL;
 
   if (dialog == NULL) {
     GtkTreeViewColumn *column;
@@ -462,7 +639,8 @@ AlbumDetails* multiple_album_dialog(GList *albums)
     g_assert (dialog != NULL);
     gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_window));
     albums_listview = glade_xml_get_widget (glade, "albums_listview");
-    /* TODO: A little hacky */
+    ok_button = glade_xml_get_widget (glade, "ok_button");
+
     g_signal_connect (albums_listview, "row-activated", G_CALLBACK (album_row_activated), dialog);
 
     albums_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
@@ -480,6 +658,8 @@ AlbumDetails* multiple_album_dialog(GList *albums)
     gtk_tree_view_set_model (GTK_TREE_VIEW (albums_listview), GTK_TREE_MODEL (albums_store));
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (albums_listview));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+    gtk_widget_set_sensitive (ok_button, FALSE);
+    g_signal_connect (selection, "changed", (GCallback)selected_album_changed, ok_button);
   }
 
   gtk_list_store_clear (albums_store);
@@ -493,11 +673,13 @@ AlbumDetails* multiple_album_dialog(GList *albums)
                         2, album,
                         -1);
   }
-  /*
-   * TODO: focus is a little broken here. The first row should be
-   * selected, so just hitting return works.
-   */
-  gtk_widget_grab_focus (albums_listview);
+
+  /* Select the first album */
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (albums_store), &iter))
+  {
+    gtk_tree_selection_select_iter (selection, &iter);
+  }
+
   gtk_widget_show_all (dialog);
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_hide (dialog);
@@ -505,15 +687,19 @@ AlbumDetails* multiple_album_dialog(GList *albums)
   if (response == GTK_RESPONSE_DELETE_EVENT) {
     return NULL;
   }
-  gtk_tree_selection_get_selected (selection, NULL, &iter);
-  gtk_tree_model_get (GTK_TREE_MODEL (albums_store), &iter, 2, &album, -1);
-  return album;
+
+  if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+    gtk_tree_model_get (GTK_TREE_MODEL (albums_store), &iter, 2, &album, -1);
+    return album;
+  } else {
+    return NULL;
+  }
 }
 
 /**
  * The GConf key for the base path changed
  */
-void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_BASEURI) == 0);
   g_free (base_uri);
@@ -528,7 +714,7 @@ void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /**
  * The GConf key for the directory pattern changed
  */
-void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_PATH_PATTERN) == 0);
   if (entry->value == NULL) {
@@ -543,7 +729,7 @@ void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *en
 /**
  * The GConf key for the filename pattern changed
  */
-void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_FILE_PATTERN) == 0);
   if (entry->value == NULL) {
@@ -558,7 +744,7 @@ void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *en
 /**
  * The GConf key for the paranoia mode has changed
  */
-void paranoia_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void paranoia_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_PARANOIA) == 0);
   if (entry->value == NULL) {
@@ -574,7 +760,7 @@ void paranoia_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /**
  * The GConf key for the strip characters option changed
  */
-void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_STRIP) == 0);
   if (entry->value == NULL) {
@@ -587,7 +773,7 @@ void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gp
 /**
  * The GConf key for the eject when finished option changed
  */
-void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_EJECT) == 0);
   if (entry->value == NULL) {
@@ -597,10 +783,23 @@ void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gp
   }
 }
 
+/**
+ * The GConf key for the eject when finished option changed
+ */
+static void open_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+  g_assert (strcmp (entry->key, GCONF_OPEN) == 0);
+  if (entry->value == NULL) {
+    open_finished = FALSE;
+  } else {
+    open_finished = gconf_value_get_bool (entry->value);
+  }
+}
+
 /**
  * The GConf key for audio volume changes
  */
-void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_AUDIO_VOLUME) == 0);
   
@@ -790,12 +989,9 @@ media_removed_cb (NautilusBurnDrive *drive,
 static void
 set_drive_from_device (const char *device)
 {
-  gboolean is_locked;
-  char *reason;
   NautilusBurnDriveMonitor *monitor;
 
   if (drive) {
-    nautilus_burn_drive_unlock (drive);
     nautilus_burn_drive_unref (drive);
     drive = NULL;
   }
@@ -823,12 +1019,6 @@ set_drive_from_device (const char *device)
     return;
   }
 
-  is_locked = nautilus_burn_drive_lock (drive, _("Extracting audio from CD"), &reason);
-  if (! is_locked) {
-    g_warning ("Could not lock drive: %s", reason);
-    g_free (reason);
-  }
-
   g_signal_connect (drive, "media-added", G_CALLBACK (media_added_cb), NULL);
   g_signal_connect (drive, "media-removed", G_CALLBACK (media_removed_cb), NULL);
 }
@@ -924,7 +1114,7 @@ prefs_get_default_device (void)
 /**
  * The GConf key for the device changed
  */
-void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   const char *device;
   gboolean ignore_no_media = GPOINTER_TO_INT (user_data);
@@ -955,15 +1145,15 @@ void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
   set_device (device, ignore_no_media);
 }
 
-void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   GMAudioProfile *profile;
   
   g_assert (strcmp (entry->key, GCONF_AUDIO_PROFILE) == 0);
   if (!entry->value) return;
-  profile = gm_audio_profile_lookup(gconf_value_get_string (entry->value));
+  profile = gm_audio_profile_lookup (gconf_value_get_string (entry->value));
   if (profile != NULL)
-    g_object_set(extractor, "profile", profile, NULL);
+    g_object_set (extractor, "profile", profile, NULL);
 
   if (profile == NULL || !sj_extractor_supports_profile(profile)) {
     GtkWidget *dialog;
@@ -990,7 +1180,7 @@ void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /**
  * Configure the http proxy
  */
-void
+static void
 http_proxy_setup (GConfClient *client)
 {
   if (!gconf_client_get_bool (client, GCONF_HTTP_PROXY_ENABLE, NULL)) {
@@ -1010,7 +1200,7 @@ http_proxy_setup (GConfClient *client)
 /**
  * The GConf key for the HTTP proxy being enabled changed.
  */
-void http_proxy_enable_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void http_proxy_enable_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_HTTP_PROXY_ENABLE) == 0);
   if (entry->value == NULL) return;
@@ -1020,7 +1210,7 @@ void http_proxy_enable_changed_cb (GConfClient *client, guint cnxn_id, GConfEntr
 /**
  * The GConf key for the HTTP proxy changed.
  */
-void http_proxy_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void http_proxy_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_HTTP_PROXY) == 0);
   if (entry->value == NULL) return;
@@ -1030,7 +1220,7 @@ void http_proxy_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entr
 /**
  * The GConf key for the HTTP proxy port changed.
  */
-void http_proxy_port_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void http_proxy_port_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_HTTP_PROXY_PORT) == 0);
   if (entry->value == NULL) return;
@@ -1189,9 +1379,11 @@ void on_title_edit_changed(GtkEditable *widget, gpointer user_data) {
 void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
   GtkTreeIter iter;
   TrackDetails *track;
+  gchar *current_track_artist, *former_album_artist=NULL;
 
   g_return_if_fail (current_album != NULL);
   if (current_album->artist) {
+    former_album_artist=g_strdup(current_album->artist);
     g_free (current_album->artist);
   }
   current_album->artist = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
@@ -1200,10 +1392,15 @@ void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
   
   /* Set the artist field in each tree row */
   do {
-    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_DETAILS, &track, -1);
-    g_free (track->artist);
-    track->artist = g_strdup (current_album->artist);
-    gtk_list_store_set (track_store, &iter, COLUMN_ARTIST, track->artist, -1);
+    gtk_tree_model_get(GTK_TREE_MODEL (track_store), &iter, COLUMN_ARTIST, &current_track_artist, -1);
+	/* Change track artist if it matched album artist before the change */
+    if ((strcasecmp (current_track_artist, former_album_artist) == 0) || (strcasecmp (current_track_artist, current_album->artist) == 0)) {
+      gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_DETAILS, &track, -1);
+      g_free (track->artist);
+      track->artist = g_strdup (current_album->artist);
+      gtk_list_store_set (track_store, &iter, COLUMN_ARTIST, track->artist, -1);
+      gtk_tree_model_get(GTK_TREE_MODEL (track_store), &iter, COLUMN_ARTIST, &current_track_artist, -1);
+	}
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(track_store), &iter));
 }
 
@@ -1215,6 +1412,23 @@ void on_genre_edit_changed(GtkEditable *widget, gpointer user_data) {
   current_album->genre = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
 }
 
+void on_year_edit_changed(GtkEditable *widget, gpointer user_data) {
+  const gchar* yearstr;
+  int year;
+  
+  g_return_if_fail (current_album != NULL);
+  
+  yearstr = gtk_entry_get_text (GTK_ENTRY (widget));
+  year = atoi (yearstr);
+  if (year > 0) {
+    if (current_album->release_date) {
+      g_date_set_dmy (current_album->release_date, 1, 1, year);
+    } else {
+      current_album->release_date = g_date_new_dmy (1, 1, year);
+    }
+  }
+}
+
 void on_contents_activate(GtkWidget *button, gpointer user_data) {
   GError *error = NULL;
 
@@ -1238,42 +1452,6 @@ void on_contents_activate(GtkWidget *button, gpointer user_data) {
   }
 }
 
-
-static const char* const genres[] = {
-  "Ambient",
-  "Blues",
-  "Classical",
-  "Country",
-  "Dance",
-  "Electronica",
-  "Folk",
-  "Funk",
-  "Jazz",
-  "Latin",
-  "Pop",
-  "Rap",
-  "Reggae",
-  "Rock",
-  "Soul",
-  "Spoken Word",
-  NULL
-};
-
-static GtkTreeModel* create_genre_list(void) {
-  GtkListStore *store;
-  const char * const *g = genres;
-
-  store = gtk_list_store_new (1, G_TYPE_STRING);
-
-  while (*g != NULL) {
-    GtkTreeIter iter;
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, *g++, -1);
-  }
-
-  return GTK_TREE_MODEL (store);
-}
-
 GtkWidget *
 sj_make_volume_button (void)
 {
@@ -1312,6 +1490,81 @@ on_message_received (const char *message, gpointer user_data)
   }
 }
 
+/**
+ * Performs various checks to ensure CD duplication is available.
+ * If this is found TRUE is returned, otherwise FALSE is returned.
+ */
+static gboolean
+is_cd_duplication_available()
+{
+  // First check the nautilus-cd-burner tool is available in the path
+  gchar* nautilus_cd_burner = g_find_program_in_path ("nautilus-cd-burner");
+  if (nautilus_cd_burner == NULL) {
+    return FALSE;
+  } 
+  g_free(nautilus_cd_burner);
+
+  // Second check the cdrdao tool is available in the path
+  gchar* cdrdao = g_find_program_in_path ("cdrdao");
+  if (cdrdao == NULL) {
+    return FALSE;
+  } 
+  g_free(cdrdao);  
+
+  // Now check that there is at least one cd recorder available
+  GList                    *drives;
+  NautilusBurnDriveMonitor *monitor;
+
+  monitor = nautilus_burn_get_drive_monitor ();
+  drives = nautilus_burn_drive_monitor_get_recorder_drives (monitor);
+
+  if (drives == NULL) {
+     return FALSE;
+  }
+
+  g_list_free (drives);
+  return TRUE;
+}
+
+/**
+ * Clicked on duplicate in the UI (button/menu)
+ */
+void on_duplicate_activate (GtkWidget *button, gpointer user_data)
+{
+  GError *error = NULL;
+  const gchar* device;
+
+  device = nautilus_burn_drive_get_device (drive);
+  if (!g_spawn_command_line_sync (g_strconcat ("nautilus-cd-burner --source-device=", device, NULL), NULL, NULL, NULL, &error)) {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_CLOSE,
+                                       "<b>%s</b>\n\n%s\n%s: %s",
+				       _("Could not duplicate disc"),
+				       _("Sound Juicer could not duplicate the disc"),
+				       _("Reason"),
+				       error->message);
+      gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      g_error_free (error);	
+  }
+}
+
+/**
+ * Sets the duplication buttons sensitive property if duplication is enabled.
+ * This is setup in the main entry point.
+ */
+static void set_duplication(gboolean enabled)
+{
+  if (duplication_enabled) {
+    gtk_widget_set_sensitive (duplicate, enabled);
+  }
+}
+
 int main (int argc, char **argv)
 {
   GnomeProgram *program;
@@ -1380,6 +1633,7 @@ int main (int argc, char **argv)
   gconf_client_add_dir (gconf_client, GCONF_ROOT, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
   gconf_client_notify_add (gconf_client, GCONF_DEVICE, device_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_EJECT, eject_changed_cb, NULL, NULL, NULL);
+  gconf_client_notify_add (gconf_client, GCONF_OPEN, open_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_BASEURI, baseuri_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_STRIP, strip_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_AUDIO_PROFILE, profile_changed_cb, NULL, NULL, NULL);
@@ -1410,6 +1664,7 @@ int main (int argc, char **argv)
   glade_xml_signal_autoconnect (glade);
 
   main_window = glade_xml_get_widget (glade, "main_window");
+  message_area_vbox = glade_xml_get_widget (glade, "message_area_vbox");
   select_all_menuitem = glade_xml_get_widget (glade, "select_all");
   deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
   submit_menuitem = glade_xml_get_widget (glade, "submit");
@@ -1417,6 +1672,7 @@ int main (int argc, char **argv)
   artist_entry = glade_xml_get_widget (glade, "artist_entry");
   duration_label = glade_xml_get_widget (glade, "duration_label");
   genre_entry = glade_xml_get_widget (glade, "genre_entry");
+  year_entry = glade_xml_get_widget (glade, "year_entry");
   track_listview = glade_xml_get_widget (glade, "track_listview");
   extract_button = glade_xml_get_widget (glade, "extract_button");
   extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
@@ -1425,6 +1681,7 @@ int main (int argc, char **argv)
   next_menuitem = glade_xml_get_widget (glade, "next_track_menuitem");
   prev_menuitem = glade_xml_get_widget (glade, "previous_track_menuitem");
   status_bar = glade_xml_get_widget (glade, "status_bar");
+  duplicate = glade_xml_get_widget (glade, "duplicate_menuitem");
 
   { /* ensure that the play/pause button's size is constant */
     GtkWidget *fake_button1, *fake_button2;
@@ -1447,14 +1704,7 @@ int main (int argc, char **argv)
     gtk_size_group_add_widget (size_group, play_button);
   }
 
-  {
-    GtkEntryCompletion *completion;
-    completion = gtk_entry_completion_new ();
-    gtk_entry_completion_set_model (completion, create_genre_list ());
-    gtk_entry_completion_set_text_column (completion, 0);
-    gtk_entry_completion_set_inline_completion (completion, TRUE);
-    gtk_entry_set_completion (GTK_ENTRY (genre_entry), completion);
-  }
+  setup_genre_entry (genre_entry);
 
   track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
   gtk_tree_view_set_model (GTK_TREE_VIEW (track_listview), GTK_TREE_MODEL (track_store));
@@ -1543,6 +1793,7 @@ int main (int argc, char **argv)
   paranoia_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PARANOIA, NULL, TRUE, NULL), NULL);
   strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
+  open_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_OPEN, NULL, TRUE, NULL), NULL);
   audio_volume_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_VOLUME, NULL, TRUE, NULL), NULL);
   if (device == NULL) {
     device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), GINT_TO_POINTER (TRUE));
@@ -1555,6 +1806,10 @@ int main (int argc, char **argv)
     return 0;
   }
 
+  // Set whether duplication of a cd is available using the nautilus-cd-burner tool
+  gtk_widget_set_sensitive (duplicate, FALSE);
+  duplication_enabled = is_cd_duplication_available();
+
   gconf_bridge_bind_window_size(gconf_bridge_get(), GCONF_WINDOW, GTK_WINDOW (main_window));
   gtk_widget_show (main_window);
   gtk_main ();
