@@ -46,7 +46,7 @@ static gfloat vol = 1.0;
 
 static GtkTreeIter current_iter;
 
-static GtkWidget *play_button, *next_menuitem, *prev_menuitem, *reread_menuitem, *seek_scale, *volume_button, *statusbar;
+static GtkWidget *play_button, *next_menuitem, *prev_menuitem, *reread_menuitem, *seek_scale, *volume_button, *statusbar,  *track_listview;
 
 /**
  * Select track number.
@@ -88,6 +88,7 @@ select_track (void)
   }
 
   cd = gst_bin_get_by_name_recurse_up (GST_BIN (pipeline), "cd-source");
+  gst_element_set_state (pipeline, GST_STATE_PAUSED);
   gst_element_seek (pipeline, 1.0, gst_format_get_by_nick ("track"), GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, seek_to_track, GST_SEEK_TYPE_NONE, -1);
   current_track = seek_to_track;
   seek_to_track = -1;
@@ -182,6 +183,12 @@ cb_hop_track (GstBus *bus, GstMessage *message, gpointer user_data)
         &current_iter, COLUMN_TITLE, &title, -1);
     sj_main_set_title (title);
     g_free (title);
+
+    /* Set the Treelist focus to the item to be extracted */
+    GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL (track_store), &current_iter); 
+    gtk_tree_view_set_cursor (GTK_TREE_VIEW (track_listview), path, NULL, TRUE);
+    gtk_tree_path_free(path);
+
     play ();
   }
 }
@@ -679,4 +686,5 @@ sj_play_init (void)
   seek_scale = glade_xml_get_widget (glade, "seek_scale");
   volume_button = glade_xml_get_widget (glade, "volume_button");
   statusbar = glade_xml_get_widget (glade, "status_bar");
+  track_listview = glade_xml_get_widget (glade, "track_listview");
 }
